/*++

Copyright (c) 2014 Advantech Corporation, All Rights Reserved

Module Name:

    gpdread.c

Abstract:   Read block from physical memory using the genport driver.


Author:    Sam Hsu July 29, 2014


Environment:

    User mode

Revision History:

 	Sam Hsu	(Advantech)			 July 29, 2014
--*/

#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include <winioctl.h>
#include "gpioctl.h"        // This defines the IOCTL constants.

void __cdecl main ( int argc, char ** argv )
{

	// The following is returned by IOCTL.  It is true if the read succeeds.
	BOOL    IoctlResult;

	// The following parameters are used in the IOCTL call
	HANDLE  hndFile;            // Handle to device, obtain from CreateFile
	ULONG   PortNumber;         // Buffer sent to the driver (Port #).
	LONG    IoctlCode;
	ULONG   DataLength;
	DWORD   ReturnedLength;     // Number of bytes returned
	TAG_PHYS_STRUCT PhysStruct;
	PCHAR 	buf = NULL;
	DWORD64 len;
	DWORD address;
	DWORD size;

	// The input buffer is a ULONG containing the port address.

	// The port data is returned in the output buffer DataBuffer;

	if ( argc < 2 )
	{
		printf ( "GpyRead physical memory start address(hex)# block length(d)#\n" );
		printf ( "                        The length of block as 0, 1, ... relative\n" );
		printf ( "                        to the base memory address assinged to the driver\n" );
		printf ( "                        by PNP manager according to the driver preferences\n" );
		printf ( "                        in the INF file.\n" );
		exit ( 1 );
	}

	hndFile = CreateFile (
	              "\\\\.\\GpdDev",                    // Open the Device "file"
	              GENERIC_READ | GENERIC_WRITE,
	              FILE_SHARE_READ | FILE_SHARE_WRITE,
	              NULL,
	              OPEN_EXISTING,
	              0,
	              NULL
	          );

	if ( hndFile == INVALID_HANDLE_VALUE )      // Was the device opened?
	{
		printf ( "Unable to open the device.\n" );
		exit ( 1 );
	}

#if 0
	sscanf ( argv[1], "%x", &address );      		 	// Get the start memory address for mapping
	sscanf ( argv[2], "%d", &size );      				// Get the length of mapping block
#else
	PhysStruct.pvPhysAddress = 0x08;					// Get the start memory address for mapping
	PhysStruct.dwPhysMemSizeInBytes = 1;				// Get the length of mapping block
#endif

	DataLength = sizeof ( &PhysStruct );
	
	IoctlResult = DeviceIoControl (
	                  hndFile,            				// Handle to device
	                  IOCTL_GPD_MAPPHYSTOLIN,          	// IO Control code for Physical Memory Read
	                  &PhysStruct,        				// Buffer to driver.
	                  sizeof ( PhysStruct ), 			// Length of buffer in bytes.
	                  &PhysStruct,        				// Buffer from driver.
	                  sizeof ( PhysStruct ),        	// Length of buffer in bytes.
	                  &ReturnedLength,    				// Bytes placed in DataBuffer.
	                  NULL                				// NULL means wait till op. completes.
	              );

	if ( IoctlResult )                          // Did the IOCTL succeed?
	{		
		if ( ReturnedLength != DataLength )
		{
			printf (
			    "Ioctl transfered %d bytes, expected %d bytes\n",
			    ReturnedLength, DataLength );
		}

		if ( (PBYTE)PhysStruct.pvPhysMemLin == NULL )
		{
			printf ( "Not successfully to mapping from physical memory 0x%x length %d\n", PhysStruct.pvPhysAddress, PhysStruct.dwPhysMemSizeInBytes );
			goto err;		
		}	
		
		len = PhysStruct.dwPhysMemSizeInBytes;
		buf = malloc(len);	
		printf ( "PhysStruct.pvPhysMemLin = 0x%x, len = %d\n", PhysStruct.pvPhysMemLin, len );
		memcpy ( buf, (PVOID) PhysStruct.pvPhysMemLin, 1 );
		
		printf ( "Successfully to mapping from physical memory 0x%x length %d returned %x\n", PhysStruct.pvPhysAddress, PhysStruct.dwPhysMemSizeInBytes, buf[0] );	
	}
	else
	{
		printf ( "Ioctl failed with code %ld\n", GetLastError() );
	}
	
	PhysStruct.dwPhysMemSizeInBytes = 100;

err:
	IoctlResult = DeviceIoControl (
	                  hndFile,            				// Handle to device
	                  IOCTL_GPD_UNMAPPHYSADDR,          // IO Control code for unmapping physical memory read
	                  &PhysStruct,        				// Buffer to driver.
	                  sizeof ( PhysStruct ), 			// Length of buffer in bytes.
	                  NULL,        						// Buffer from driver.
	                  0,         						// Length of buffer in bytes.
	                  &ReturnedLength,    				// Bytes placed in DataBuffer.
	                  NULL                				// NULL means wait till op. completes.
	              );            

	if ( !CloseHandle ( hndFile ) )             // Close the Device "file".
	{
		printf ( "Failed to close device.\n" );
	}
	
	exit ( 0 );
}

